==> slapd-cyrus-0.01.tbz: lib/perl5/site_perl/5.10.1/Slapd/Backend/Cyrus/Mail.pm <==
=head1 NAME

Slapd::Backend::Cyrus::Mail - mail's LDAP slapd backend class

=head1 DESCRIPTION

Synchronization mail shared lists between LDAP and local database.

=cut

package Slapd::Backend::Cyrus::Mail;

use utf8;
use strict;
use warnings;
use base 'Slapd::Backend::Cyrus::Base';

use BerkeleyDB;
use List::MoreUtils qw( any uniq );
use Net::LDAP::Constant qw/
	LDAP_SUCCESS
	LDAP_NO_SUCH_OBJECT
	LDAP_UNWILLING_TO_PERFORM
	LDAP_ALREADY_EXISTS
/;

use constant stored_entries => qw/domain folder acl/;
use constant stored_attributes => qw/dc cn memberUid/;
use constant key_attribute => qw/dc cn/;

use constant skip_users => qw/anyone/;

__PACKAGE__->mk_accessors(qw/ acl_db /);

=head1 PROTECTED METHODS

=over

=item B<_init>()

Purpose:
  * Initialization

Arguments:
  * None

Returns:
  * Nonzero if initialization failed

=cut

sub _init
{
	my $self = shift;
	my $code = $self->SUPER::_init();
	return $code unless $code == LDAP_SUCCESS;

	$self->acl_db = new BerkeleyDB::Hash(
		-Env      => $self->env,
		-Filename => "acl.db",
		-Flags    => DB_CREATE,
	) or die "Can't open file acl.db: $! $BerkeleyDB::Error\n";

	return LDAP_SUCCESS;
}


=item B<_search>( filter, entries )

Purpose:
  * Search

Arguments:
  * filter : Hashref - search filter
  * entries : Arrayref of Net::LDAP::Entry - search in these entries

Returns:
  * Scalar - result code
  * Array  - Net::LDAP::Entry entries

=cut

sub _search
{
	my $self = shift;
	my %in = @_;

	my %filter = $in{filter} ? %{$in{filter}} : ();
	my @entries = @{$in{entries}};

	my @match_entries;

	unless( any { $_ ne 'dc' } keys %filter )
	{
		foreach my $entry ( grep { $self->_is_domain( $_ ) } @entries )
		{
			push @match_entries, $entry if $self->_domain_exists( $entry );
		}
	}

	unless( any { $_ ne 'cn' } keys %filter )
	{
		foreach my $entry ( grep { $self->_is_folder( $_ ) } @entries )
		{
			push @match_entries, $entry
				if $self->cyradm->listmailbox( $self->_mailbox( $entry ) );
			my $error = $self->_cyradm_error();
			return $error if $error != LDAP_SUCCESS;
		}
	}

	unless( any { $_ ne 'cn' && $_ ne 'memberUid' } keys %filter )
	{
		foreach my $entry ( grep { $self->_is_acl( $_ ) } @entries )
		{
			my $mailbox = $self->_mailbox( $entry );
			if( my %acls = $self->cyradm->listaclmailbox( $mailbox ) )
			{
				my @users;
				foreach my $user ( grep { my $u = $_; !( any { $u eq $_ } $self->skip_users ) } keys %acls )
				{
					push @users, $user if $self->_has_acl( $mailbox, $user, $entry->get_value('cn') )
						&& $self->_check_acl( $entry->get_value('cn'), $acls{$user} );
				}
				if( !exists $filter{memberUid} || $filter{memberUid} ~~ \@users )
				{
					my $full_entry = $entry->clone();
					$full_entry->replace( memberUid => \@users );
					push @match_entries, $full_entry;
				}
			}
			else
			{
				my $error = $self->_cyradm_error();
				return $error if $error != LDAP_SUCCESS;
			}
		}
	}

	return ( LDAP_SUCCESS, @match_entries );
}

=item B<_modify>( entry, actions )

Purpose:
  * Modify

Arguments:
  * entry : Net::LDAP::Entry - compared entry
  * actions : ArrayRef - list of actions

Returns:
  * Scalar - result code

=cut

sub _modify
{
	my $self = shift;
	my %in = @_;

	my $entry = $in{entry};
	my @actions = @{$in{actions}};

	foreach my $action ( @actions )
	{
		if( $self->_is_acl( $entry ) && $action->{attr} eq 'memberUid' )
		{
			my $mailbox = $self->_mailbox( $entry );
			my @add = $action->{action} ~~ ['ADD', 'REPLACE'] ? @{$action->{values}} : ();
			my @del = grep { !( $_ ~~ \@add ) }
				$action->{action} eq 'DELETE' ? @{$action->{values}}
				: $action->{action} eq 'REPLACE' ? $entry->get_value('memberUid')
				: ();
			foreach my $act ( qw/del add/ )
			{
				foreach my $user ( $act eq 'add' ? @add : @del )
				{
					my $code = $self->_apply_acl( $mailbox, $user, $entry, $act );
					return $code if $code != LDAP_SUCCESS;
				}
			}
		}
		else
		{
			return LDAP_UNWILLING_TO_PERFORM;
		}
	}

	return LDAP_SUCCESS;
}

=item B<_add>( entry )

Purpose:
  * Add

Arguments:
  * entry : Net::LDAP::Entry - added entry

Returns:
  * Scalar - result code

=cut

sub _add
{
	my $self = shift;
	my %in = @_;

	my $entry = $in{entry};

	if( $self->_is_domain( $entry ) )
	{
		my $status = $self->domain_db->db_put( $entry->get_value('dc'), 1 );
		warn $BerkeleyDB::Error unless $status == 0;

		my $cursor = $self->domainuser_db->db_cursor() or die $BerkeleyDB::Error;
		my $domain = $entry->get_value('dc');
		my $user = '';
		my $mail = '';
		my $flag = DB_SET;
		while( ( $status = $cursor->c_pget( $domain, $user, $mail, $flag ) ) == 0 )
		{
			my $mailbox = "user/$user\@$domain";
			unless( $self->cyradm->listmailbox( $mailbox ) )
			{
				unless( $self->cyradm->createmailbox( $mailbox ) )
				{
					my $error = $self->_cyradm_error();
					return $error if $error != LDAP_ALREADY_EXISTS;
				}
			}
			$flag = DB_NEXT_DUP;
		}
		warn "$status ($BerkeleyDB::Error)" unless $status ~~ [0, DB_NOTFOUND];
		$cursor->c_close();
	}
	elsif( $self->_is_folder( $entry ) )
	{
		my $mailbox = $self->_mailbox( $entry );
		unless( $self->cyradm->createmailbox( $mailbox ) )
		{
			my $error = $self->_cyradm_error();
			return $error if $error != LDAP_ALREADY_EXISTS;
		}
		my %acls = $self->cyradm->listaclmailbox( $mailbox );
		foreach my $user ( grep { !( $_ ~~ [ $self->skip_users ] ) } keys %acls )
		{
			$self->cyradm->setaclmailbox( $mailbox, $user, '' );
		}
	}
	elsif( $self->_is_acl( $entry ) )
	{
		my $mailbox = $self->_mailbox( $entry );
		my $acl = $entry->get_value('cn');
		my $error;
		foreach my $user ( $entry->get_value('memberUid') )
		{
			my $code = $self->_apply_acl( $mailbox, $user, $entry, 'add' );
			return $code if $code != LDAP_SUCCESS;
		}
		return $error if $error && $error != LDAP_SUCCESS;
	}
	else
	{
		return LDAP_UNWILLING_TO_PERFORM;
	}

	return LDAP_SUCCESS;
}

=item B<_delete>( entry )

Purpose:
  * Delete

Arguments:
  * entry : Net::LDAP::Entry - entry

Returns:
  * None

=cut

sub _delete
{
	my $self = shift;
	my %in = @_;

	my $entry = $in{entry};

	if( $self->_is_domain( $entry ) )
	{
		my $status = $self->domain_db->db_del( $entry->get_value('dc') );
		warn $BerkeleyDB::Error unless $status ~~ [0, DB_NOTFOUND];
	}
	elsif( $self->_is_folder( $entry ) )
	{
		# do nothing
	}
	elsif( $self->_is_acl( $entry ) )
	{
		my $mailbox = $self->_mailbox( $entry );
		foreach my $user ( $entry->get_value('memberUid') )
		{
			my $code = $self->_apply_acl( $mailbox, $user, $entry, 'del' );
			return $code if $code != LDAP_SUCCESS;
		}
	}
	else
	{
		return LDAP_UNWILLING_TO_PERFORM;
	}

	return LDAP_SUCCESS;
}

=item B<_config>( $name, $value )

Purpose:
  * Config

Arguments:
  * Scalar - name
  * Scalar - value

Returns:
  * Scalar - result code

=cut

sub _config
{
	my $self = shift;
	my( $name, $value ) = @_;
	if( $name eq 'cyrusDomain' )
	{
		$self->{domain_filter} = $self->parse_filter( $value );
		return LDAP_SUCCESS;
	}
	elsif( $name eq 'cyrusFolder' )
	{
		$self->{folder_filter} = $self->parse_filter( $value );
		return LDAP_SUCCESS;
	}
	elsif( $name eq 'cyrusAcl' )
	{
		$self->{acl_filter} = $self->parse_filter( $value );
		return LDAP_SUCCESS;
	}
	else
	{
		return $self->SUPER::_config(@_);
	}
}

=back

=head1 PRIVATE METHODS

=over

=item B<_is_domain>( $entry )

Purpose:
  * Check if C<$entry> is domain

Arguments:
  * Net::LDAP::Entry object - entry

Returns:
  * Scalar - boolean

=cut

sub _is_domain
{
	my( $self, $entry ) = @_;
	return $self->check_filter( $entry, $self->{domain_filter} );
}

=item B<_is_folder>( $entry )

Purpose:
  * Check if C<$entry> is folder

Arguments:
  * Net::LDAP::Entry object - entry

Returns:
  * Scalar - boolean

=cut

sub _is_folder
{
	my( $self, $entry ) = @_;
	return $self->check_filter( $entry, $self->{folder_filter} );
}

=item B<_is_acl>( $entry )

Purpose:
  * Check if C<$entry> is folder's acl

Arguments:
  * Net::LDAP::Entry object - entry

Returns:
  * Scalar - boolean

=cut

sub _is_acl
{
	my( $self, $entry ) = @_;
	return $self->check_filter( $entry, $self->{acl_filter} );
}

=item B<_mailbox>( $entry )

Purpose:
  * Get mailbox of C<$entry>

Arguments:
  * Net::LDAP::Entry object - entry

Returns:
  * Scalar - mailbox

=cut

sub _mailbox
{
	my( $self, $entry ) = @_;
	my @dn = $self->parse_dn( $entry->dn() );
	shift @dn and shift @dn if $self->_is_acl( $entry );
	my( @folder, $domain );
	while( @dn )
	{
		my $key = shift @dn;
		my $value = shift @dn;
		if( $key eq 'cn' )
		{
			unshift @folder, $value;
		}
		elsif( $key eq 'dc' )
		{
			$domain = $value;
		}
	}
	unshift @folder, 'shared';
	return join( $self->{sep}, @folder ) . '@' . $domain;
}

=item B<_domain_exists>( $entry )

Purpose:
  * Check if domain exists for C<$entry>

Arguments:
  * Net::LDAP::Entry object - entry

Returns:
  * Scalar - boolean

=cut

sub _domain_exists
{
	my( $self, $entry ) = @_;
	my $exists;
	my $status = $self->domain_db->db_get( $entry->get_value('dc'), $exists );
	return $status == 0 && $exists;
}

=item B<_encode_acl>( $acl )

Purpose:
  * Encode acl

Arguments:
  * Scalar - acl string in human form

Returns:
  * Scalar - acl string

=cut

sub _encode_acl
{
	my( $self, $acl ) = @_;
	my %aclalias = (
		none   => '',
		read   => 'lrs',
		post   => 'lrsp',
		append => 'lrsip',
		write  => 'lrswipkxte',
		delete => 'lrxte',
		all    => 'lrswipkxtea',
	);
	return exists $aclalias{$acl} ? $aclalias{$acl} : $acl;
}

=item B<_check_acl>( $acl1, $acl2 )

Purpose:
  * Checks if C<$acl1> is contained in C<$acl2>

Arguments:
  * Scalar - acl contained in
  * Scalar - acl contains

Returns:
  * Scalar - boolean

=cut

sub _check_acl
{
	my( $self, $acl1, $acl2 ) = @_;
	$acl1 = $self->_encode_acl( $acl1 );
	$acl2 = $self->_encode_acl( $acl2 );
	my @acl1 = split '', $acl1;
	my @acl2 = split '', $acl2;
	return @acl1 == grep { $_ ~~ \@acl1 } @acl2;
}

=item B<_has_acl>( $mailbox, $user, $acl )

Purpose:
  * Checks db if C<$acl> exists for C<$mailbox> $C<$user>

Arguments:
  * Scalar - mailbox
  * Scalar - user
  * Scalar = acl

Returns:
  * Scalar - boolean

=cut

sub _has_acl
{
	my( $self, $mailbox, $user, $acl ) = @_;
	my $db_acl = '';
	my $status = $self->acl_db->db_get( "$mailbox $user", $db_acl );
	warn $status unless $status ~~ [0, DB_NOTFOUND];
	return $acl ~~ [ split /\s+/, $db_acl ];
}

=item B<_apply_acl>( $mailbox, $user, $entry, $act )

Purpose:
  * Apply acl to mailbox

Arguments:
  * Scalar - mailbox
  * Scalar - user
  * Net::LDAP::Entry - acl entry
  * Scalar - action ('add' or 'del')

Returns:
  * Scalar - ldap result code

=cut

sub _apply_acl
{
	my( $self, $mailbox, $user, $entry, $act ) = @_;

	my $db_acl = '';
	my $status = $self->acl_db->db_get( "$mailbox $user", $db_acl );
	warn $status unless $status ~~ [0, DB_NOTFOUND];
	my %acls = map { $_ => 1 } split /\s+/, $db_acl;

	if( $act eq 'add' )
	{
		$acls{ $entry->get_value('cn') } = 1;
	}
	elsif( $act eq 'del' )
	{
		delete $acls{ $entry->get_value('cn') };
	}
	my $acl = join '', uniq( split '', join '', map $self->_encode_acl($_), keys %acls );
	unless( $self->cyradm->setaclmailbox( $mailbox, $user, $acl ) )
	{
		return $self->_cyradm_error();
	}

	$db_acl = join ' ', sort keys %acls;
	my $status = $self->acl_db->db_put( "$mailbox $user", $db_acl );
	warn $status unless $status == 0;

	return LDAP_SUCCESS;
}

=back

=cut

1;

=head1 SEE ALSO

L<Slapd::Backend::Cyrus::Base>, L<Slapd::Backend::Cyrus>, L<Slapd::Backend::Proxy>, L<slapd-perl>

=head1 AUTHOR

Aleksey Mashanov <aleksey.mashanov@gmail.com>

=head1 LICENSE

This library is free software, you can redistribute it and/or modify it under
the same terms as Perl itself.

=cut

==> slapd-cyrus-1.00.tbz: lib/perl5/site_perl/5.12.4/Slapd/Backend/Cyrus/Mail.pm <==
=head1 NAME

Slapd::Backend::Cyrus::Mail - mail's LDAP slapd backend class

=head1 DESCRIPTION

Synchronization mail shared lists between LDAP and local database.

=cut

package Slapd::Backend::Cyrus::Mail;

use utf8;
use strict;
use warnings;
use base 'Slapd::Backend::Cyrus::Base';

use BerkeleyDB;
use List::MoreUtils qw( any uniq );
use Net::LDAP::Constant qw/
	LDAP_SUCCESS
	LDAP_NO_SUCH_OBJECT
	LDAP_UNWILLING_TO_PERFORM
	LDAP_ALREADY_EXISTS
/;

use constant stored_entries => qw/domain folder acl/;
use constant stored_attributes => qw/dc cn memberUid/;
use constant key_attribute => qw/dc cn/;

use constant skip_users => qw/anyone/;

__PACKAGE__->mk_accessors(qw/ acl_db /);

=head1 PROTECTED METHODS

=over

=item B<_init>()

Purpose:
  * Initialization

Arguments:
  * None

Returns:
  * Nonzero if initialization failed

=cut

sub _init
{
	my $self = shift;
	my $code = $self->SUPER::_init();
	return $code unless $code == LDAP_SUCCESS;

	$self->acl_db = new BerkeleyDB::Hash(
		-Env      => $self->env,
		-Filename => "acl.db",
		-Flags    => DB_CREATE,
	) or die "Can't open file acl.db: $! $BerkeleyDB::Error\n";

	return LDAP_SUCCESS;
}


=item B<_search>( filter, entries )

Purpose:
  * Search

Arguments:
  * filter : Hashref - search filter
  * entries : Arrayref of Net::LDAP::Entry - search in these entries

Returns:
  * Scalar - result code
  * Array  - Net::LDAP::Entry entries

=cut

sub _search
{
	my $self = shift;
	my %in = @_;

	my %filter = $in{filter} ? %{$in{filter}} : ();
	my @entries = @{$in{entries}};

	my @match_entries;

	unless( any { $_ ne 'dc' } keys %filter )
	{
		foreach my $entry ( grep { $self->_is_domain( $_ ) } @entries )
		{
			push @match_entries, $entry if $self->_domain_exists( $entry );
		}
	}

	unless( any { $_ ne 'cn' } keys %filter )
	{
		foreach my $entry ( grep { $self->_is_folder( $_ ) } @entries )
		{
			push @match_entries, $entry
				if $self->cyradm->listmailbox( $self->_mailbox( $entry ) );
			my $error = $self->_cyradm_error();
			return $error if $error != LDAP_SUCCESS;
		}
	}

	unless( any { $_ ne 'cn' && $_ ne 'memberUid' } keys %filter )
	{
		foreach my $entry ( grep { $self->_is_acl( $_ ) } @entries )
		{
			my $mailbox = $self->_mailbox( $entry );
			if( my %acls = $self->cyradm->listaclmailbox( $mailbox ) )
			{
				my @users;
				foreach my $user ( grep { my $u = $_; !( any { $u eq $_ } $self->skip_users ) } keys %acls )
				{
					push @users, $user if $self->_has_acl( $mailbox, $user, $entry->get_value('cn') )
						&& $self->_check_acl( $entry->get_value('cn'), $acls{$user} );
				}
				if( !exists $filter{memberUid} || $filter{memberUid} ~~ \@users )
				{
					my $full_entry = $entry->clone();
					$full_entry->replace( memberUid => \@users );
					push @match_entries, $full_entry;
				}
			}
			else
			{
				my $error = $self->_cyradm_error();
				return $error if $error != LDAP_SUCCESS;
			}
		}
	}

	return ( LDAP_SUCCESS, @match_entries );
}

=item B<_modify>( entry, actions )

Purpose:
  * Modify

Arguments:
  * entry : Net::LDAP::Entry - compared entry
  * actions : ArrayRef - list of actions

Returns:
  * Scalar - result code

=cut

sub _modify
{
	my $self = shift;
	my %in = @_;

	my $entry = $in{entry};
	my @actions = @{$in{actions}};

	foreach my $action ( @actions )
	{
		if( $self->_is_acl( $entry ) && $action->{attr} eq 'memberUid' )
		{
			my $mailbox = $self->_mailbox( $entry );
			my @add = $action->{action} ~~ ['ADD', 'REPLACE'] ? @{$action->{values}} : ();
			my @del = grep { !( $_ ~~ \@add ) }
				$action->{action} eq 'DELETE' ? @{$action->{values}}
				: $action->{action} eq 'REPLACE' ? $entry->get_value('memberUid')
				: ();
			foreach my $act ( qw/del add/ )
			{
				foreach my $user ( $act eq 'add' ? @add : @del )
				{
					my $code = $self->_apply_acl( $mailbox, $user, $entry, $act );
					return $code if $code != LDAP_SUCCESS;
				}
			}
		}
		else
		{
			return LDAP_UNWILLING_TO_PERFORM;
		}
	}

	return LDAP_SUCCESS;
}

=item B<_add>( entry )

Purpose:
  * Add

Arguments:
  * entry : Net::LDAP::Entry - added entry

Returns:
  * Scalar - result code

=cut

sub _add
{
	my $self = shift;
	my %in = @_;

	my $entry = $in{entry};

	if( $self->_is_domain( $entry ) )
	{
		my $status = $self->domain_db->db_put( $entry->get_value('dc'), 1 );
		warn $BerkeleyDB::Error unless $status == 0;

		my $cursor = $self->domainuser_db->db_cursor() or die $BerkeleyDB::Error;
		my $domain = $entry->get_value('dc');
		my $user = '';
		my $mail = '';
		my $flag = DB_SET;
		while( ( $status = $cursor->c_pget( $domain, $user, $mail, $flag ) ) == 0 )
		{
			my $mailbox = "user/$user\@$domain";
			unless( $self->cyradm->listmailbox( $mailbox ) )
			{
				unless( $self->cyradm->createmailbox( $mailbox ) )
				{
					my $error = $self->_cyradm_error();
					return $error if $error != LDAP_ALREADY_EXISTS;
				}
			}
			$flag = DB_NEXT_DUP;
		}
		warn "$status ($BerkeleyDB::Error)" unless $status ~~ [0, DB_NOTFOUND];
		$cursor->c_close();
	}
	elsif( $self->_is_folder( $entry ) )
	{
		my $mailbox = $self->_mailbox( $entry );
		unless( $self->cyradm->createmailbox( $mailbox ) )
		{
			my $error = $self->_cyradm_error();
			return $error if $error != LDAP_ALREADY_EXISTS;
		}
		my %acls = $self->cyradm->listaclmailbox( $mailbox );
		foreach my $user ( grep { !( $_ ~~ [ $self->skip_users ] ) } keys %acls )
		{
			$self->cyradm->setaclmailbox( $mailbox, $user, '' );
		}
	}
	elsif( $self->_is_acl( $entry ) )
	{
		my $mailbox = $self->_mailbox( $entry );
		my $acl = $entry->get_value('cn');
		my $error;
		foreach my $user ( $entry->get_value('memberUid') )
		{
			my $code = $self->_apply_acl( $mailbox, $user, $entry, 'add' );
			return $code if $code != LDAP_SUCCESS;
		}
		return $error if $error && $error != LDAP_SUCCESS;
	}
	else
	{
		return LDAP_UNWILLING_TO_PERFORM;
	}

	return LDAP_SUCCESS;
}

=item B<_delete>( entry )

Purpose:
  * Delete

Arguments:
  * entry : Net::LDAP::Entry - entry

Returns:
  * None

=cut

sub _delete
{
	my $self = shift;
	my %in = @_;

	my $entry = $in{entry};

	if( $self->_is_domain( $entry ) )
	{
		my $status = $self->domain_db->db_del( $entry->get_value('dc') );
		warn $BerkeleyDB::Error unless $status ~~ [0, DB_NOTFOUND];
	}
	elsif( $self->_is_folder( $entry ) )
	{
		# do nothing
	}
	elsif( $self->_is_acl( $entry ) )
	{
		my $mailbox = $self->_mailbox( $entry );
		foreach my $user ( $entry->get_value('memberUid') )
		{
			my $code = $self->_apply_acl( $mailbox, $user, $entry, 'del' );
			return $code if $code != LDAP_SUCCESS;
		}
	}
	else
	{
		return LDAP_UNWILLING_TO_PERFORM;
	}

	return LDAP_SUCCESS;
}

=item B<_config>( $name, $value )

Purpose:
  * Config

Arguments:
  * Scalar - name
  * Scalar - value

Returns:
  * Scalar - result code

=cut

sub _config
{
	my $self = shift;
	my( $name, $value ) = @_;
	if( $name eq 'cyrusDomain' )
	{
		$self->{domain_filter} = $self->parse_filter( $value );
		return LDAP_SUCCESS;
	}
	elsif( $name eq 'cyrusFolder' )
	{
		$self->{folder_filter} = $self->parse_filter( $value );
		return LDAP_SUCCESS;
	}
	elsif( $name eq 'cyrusAcl' )
	{
		$self->{acl_filter} = $self->parse_filter( $value );
		return LDAP_SUCCESS;
	}
	else
	{
		return $self->SUPER::_config(@_);
	}
}

=back

=head1 PRIVATE METHODS

=over

=item B<_is_domain>( $entry )

Purpose:
  * Check if C<$entry> is domain

Arguments:
  * Net::LDAP::Entry object - entry

Returns:
  * Scalar - boolean

=cut

sub _is_domain
{
	my( $self, $entry ) = @_;
	return $self->check_filter( $entry, $self->{domain_filter} );
}

=item B<_is_folder>( $entry )

Purpose:
  * Check if C<$entry> is folder

Arguments:
  * Net::LDAP::Entry object - entry

Returns:
  * Scalar - boolean

=cut

sub _is_folder
{
	my( $self, $entry ) = @_;
	return $self->check_filter( $entry, $self->{folder_filter} );
}

=item B<_is_acl>( $entry )

Purpose:
  * Check if C<$entry> is folder's acl

Arguments:
  * Net::LDAP::Entry object - entry

Returns:
  * Scalar - boolean

=cut

sub _is_acl
{
	my( $self, $entry ) = @_;
	return $self->check_filter( $entry, $self->{acl_filter} );
}

=item B<_mailbox>( $entry )

Purpose:
  * Get mailbox of C<$entry>

Arguments:
  * Net::LDAP::Entry object - entry

Returns:
  * Scalar - mailbox

=cut

sub _mailbox
{
	my( $self, $entry ) = @_;
	my @dn = $self->parse_dn( $entry->dn() );
	shift @dn and shift @dn if $self->_is_acl( $entry );
	my( @folder, $domain );
	while( @dn )
	{
		my $key = shift @dn;
		my $value = shift @dn;
		if( $key eq 'cn' )
		{
			unshift @folder, $value;
		}
		elsif( $key eq 'dc' )
		{
			$domain = $value;
			last;
		}
	}
	unshift @folder, 'shared';
	return join( $self->{sep}, @folder ) . '@' . $domain;
}

=item B<_domain_exists>( $entry )

Purpose:
  * Check if domain exists for C<$entry>

Arguments:
  * Net::LDAP::Entry object - entry

Returns:
  * Scalar - boolean

=cut

sub _domain_exists
{
	my( $self, $entry ) = @_;
	my $exists;
	my $status = $self->domain_db->db_get( $entry->get_value('dc'), $exists );
	return $status == 0 && $exists;
}

=item B<_encode_acl>( $acl )

Purpose:
  * Encode acl

Arguments:
  * Scalar - acl string in human form

Returns:
  * Scalar - acl string

=cut

sub _encode_acl
{
	my( $self, $acl ) = @_;
	my %aclalias = (
		none   => '',
		read   => 'lrs',
		post   => 'lrsp',
		append => 'lrsip',
		write  => 'lrswipkxte',
		delete => 'lrxte',
		all    => 'lrswipkxtea',
	);
	return exists $aclalias{$acl} ? $aclalias{$acl} : $acl;
}

=item B<_check_acl>( $acl1, $acl2 )

Purpose:
  * Checks if C<$acl1> is contained in C<$acl2>

Arguments:
  * Scalar - acl contained in
  * Scalar - acl contains

Returns:
  * Scalar - boolean

=cut

sub _check_acl
{
	my( $self, $acl1, $acl2 ) = @_;
	$acl1 = $self->_encode_acl( $acl1 );
	$acl2 = $self->_encode_acl( $acl2 );
	my @acl1 = split '', $acl1;
	my @acl2 = split '', $acl2;
	return @acl1 == grep { $_ ~~ \@acl1 } @acl2;
}

=item B<_has_acl>( $mailbox, $user, $acl )

Purpose:
  * Checks db if C<$acl> exists for C<$mailbox> $C<$user>

Arguments:
  * Scalar - mailbox
  * Scalar - user
  * Scalar = acl

Returns:
  * Scalar - boolean

=cut

sub _has_acl
{
	my( $self, $mailbox, $user, $acl ) = @_;
	my $db_acl = '';
	my $status = $self->acl_db->db_get( "$mailbox $user", $db_acl );
	warn $status unless $status ~~ [0, DB_NOTFOUND];
	return $acl ~~ [ split /\s+/, $db_acl ];
}

=item B<_apply_acl>( $mailbox, $user, $entry, $act )

Purpose:
  * Apply acl to mailbox

Arguments:
  * Scalar - mailbox
  * Scalar - user
  * Net::LDAP::Entry - acl entry
  * Scalar - action ('add' or 'del')

Returns:
  * Scalar - ldap result code

=cut

sub _apply_acl
{
	my( $self, $mailbox, $user, $entry, $act ) = @_;

	my $db_acl = '';
	my $status = $self->acl_db->db_get( "$mailbox $user", $db_acl );
	warn $status unless $status ~~ [0, DB_NOTFOUND];
	my %acls = map { $_ => 1 } split /\s+/, $db_acl;

	if( $act eq 'add' )
	{
		$acls{ $entry->get_value('cn') } = 1;
	}
	elsif( $act eq 'del' )
	{
		delete $acls{ $entry->get_value('cn') };
	}
	my $acl = join '', uniq( split '', join '', map $self->_encode_acl($_), keys %acls );
	unless( $self->cyradm->setaclmailbox( $mailbox, $user, $acl ) )
	{
		return $self->_cyradm_error();
	}

	$db_acl = join ' ', sort keys %acls;
	$status = $self->acl_db->db_put( "$mailbox $user", $db_acl );
	warn $status unless $status == 0;

	return LDAP_SUCCESS;
}

=back

=cut

1;

=head1 SEE ALSO

L<Slapd::Backend::Cyrus::Base>, L<Slapd::Backend::Cyrus>, L<Slapd::Backend::Proxy>, L<slapd-perl>

=head1 AUTHOR

Aleksey Mashanov <aleksey.mashanov@gmail.com>

=head1 LICENSE

This library is free software, you can redistribute it and/or modify it under
the same terms as Perl itself.

=cut
